﻿

Partial Class DtsDaten

	Public IncomeRecord As TageseinnahmeRow
	Public CustomersRecord As TageseinnahmeRow
    Public ReadOnly ProductGroupRecords As New List(Of WarengruppeEinnahmeRow)
    Public ReadOnly MonthlyGroupRecords As New List(Of Tuple(Of Date, WarengruppeRow, Integer))
    Public MonthlyIncomeRecord As Tuple(Of Date, Integer)
    Public MonthlyCustomersRecord As Tuple(Of Date, Integer)
    Public PCSRecord As Tuple(Of Date, Double)

	Public Sub LoadDailyRecords()
		IncomeRecord = Tageseinnahme.MaxBy(Function(x) x.Tageseinnahme)
		CustomersRecord = Tageseinnahme.MaxBy(Function(x) x.Kundenzahl)
		Dim rekorde = From dist In WarengruppeEinnahme Group By dist.WarengruppeID Into rekord = MaxBy(dist.Warengruppeneinnahme) Select rekord
		ProductGroupRecords.Clear() : ProductGroupRecords.AddRange(rekorde)
	End Sub
	Private Sub CalculateMonthlyGroupRecords()
		Dim tuples = New List(Of Tuple(Of Date, WarengruppeRow, Integer))
		Dim kategorieInfos = Aggregate dist In WarengruppeEinnahme Group By kategorie = dist.WarengruppeRow Into distsOfKategorie = Group Into ToArray
		For Each info In kategorieInfos
			Dim monthInfos = Aggregate dist In info.distsOfKategorie
								Let dt = dist.TageseinnahmeRow.Datum, dtMonth = New Date(dt.Year, dt.Month, 1)
									Group By dtMonth Into monthlyDists = ToArray(dist)
										Select dtMonth, monthSum = monthlyDists.Sum(Function(x) x.Warengruppeneinnahme) Into ToArray
			Dim maxMonthPerKategorie = monthInfos.MaxBy(Function(x) x.monthSum)
			tuples.Add(Tuple.Create(maxMonthPerKategorie.dtMonth, info.kategorie, maxMonthPerKategorie.monthSum))
		Next
		Dim orderedTuples = From kat In Warengruppe Join tpl In tuples On kat Equals tpl.Item2 Select tpl
		Me.MonthlyGroupRecords.Clear()
		Me.MonthlyGroupRecords.AddRange(orderedTuples)
	End Sub
	Public Sub CalculateMonthlyRecords()
		Dim monthInfos = Aggregate tag In Tageseinnahme
					Let dt = tag.Datum, dtMonth = New Date(dt.Year, dt.Month, 1)
						Group By dtMonth Into Income = [Select](tag.Tageseinnahme), customers = [Select](tag.Kundenzahl)
							Select dtMonth, IncomeSum = Income.Sum, CustomerSum = customers.Sum Into ToArray
		Dim maxIncomeMonth = monthInfos.MaxBy(Function(x) x.IncomeSum)
		Dim maxCustomerMonth = monthInfos.MaxBy(Function(x) x.CustomerSum)
		MonthlyIncomeRecord = Tuple.Create(maxIncomeMonth.dtMonth, maxIncomeMonth.IncomeSum)
		MonthlyCustomersRecord = Tuple.Create(maxCustomerMonth.dtMonth, maxCustomerMonth.CustomerSum)

		Dim PCS As Double
		For Each row In monthInfos
			Dim TempPcs = row.IncomeSum / 100 / row.CustomerSum
			If TempPcs > PCS Then
				PCS = TempPcs
				PCSRecord = Tuple.Create(row.dtMonth, PCS)
			End If
		Next

		CalculateMonthlyGroupRecords()
	End Sub
	Public Sub FindNewStatisticRecordsDaily(Customers As Integer, DailyIncome As Double, ArrDailyGroupIncome() As Double, Datum As Date)
		Dim NewRecordFound = False
		If Not CustomersRecord Is Nothing Then
			If Customers > CustomersRecord.Kundenzahl Then
				ShowMessageform("Tageskundenrekord", CustomersRecord.Kundenzahl.ToString("#,###"), Customers.ToString("#,###"), Datum.ToShortDateString)
				NewRecordFound = True
			End If
		Else
			NewRecordFound = True
		End If
		If Not IncomeRecord Is Nothing Then
			If DailyIncome > IncomeRecord.Tageseinnahme / 100 Then
				ShowMessageform("Tageseinnahmerekord", IncomeRecord.Tageseinnahme.ToString("#,##0.00 €"), DailyIncome.ToString("#,##0.00 €"), Datum.ToShortDateString)
				NewRecordFound = True
			End If
		Else
			NewRecordFound = True
		End If
		Dim i = -1
		For Each record In ProductGroupRecords
			i += 1
			Dim ParentProductGroupRow = record.WarengruppeRow
			Dim ParentDailyIncomeRow = record.TageseinnahmeRow
			Dim RecordDate = ParentDailyIncomeRow.Datum
			Dim ProductGroupName = ParentProductGroupRow.Name
			Dim ProductGroupIncome = record.Warengruppeneinnahme / 100
			If ArrDailyGroupIncome(i) > ProductGroupIncome Then
				ShowMessageform("Tagesumsatzrekord " & ProductGroupName, ProductGroupIncome.ToString("#,##0.00 €"), ArrDailyGroupIncome(i).ToString("#,##0.00 €"), RecordDate.ToShortDateString)
				NewRecordFound = True
			End If
		Next
		If NewRecordFound Then LoadDailyRecords()
	End Sub
	Private Sub ShowMessageform(NameOfRecord As String, OldRecord As String, NewRecord As String, OldDatum As String)
		ShowMessageBox("Herzlichen Glückwunsch! Es gibt einen neuen " & NameOfRecord & Environment.NewLine &
							"alt: " & OldRecord & " vom " & OldDatum & Environment.NewLine &
							"neu: " & NewRecord)
	End Sub
	Public Sub FindNewStatisticRecordsMonthly(Datum As Date)
		Dim NewRecordFound = False
		Dim Startdate = New Date(Datum.Year, Datum.Month, 1)
		Dim MonthIncome As Double
		Dim MonthCustomers As Integer
		Dim productGroups = Warengruppe.ToArray
		Dim groupcount = productGroups.Length
		Dim ArrProductGroupIncome(0 To groupcount - 1) As Double
		Dim dt1 = Startdate.AddMonths(1)
		Dim WaregroupIncomes = From Waregrouptbl In productGroups.SelectMany(Function(x) x.GetWarengruppeEinnahmeRows)
							   Let dt = Waregrouptbl.TageseinnahmeRow.Datum
							   Where dt >= Startdate AndAlso dt < dt1 Select Waregrouptbl
		For Each rwWaregroupIncome In WaregroupIncomes
			Dim rwPG = rwWaregroupIncome.WarengruppeRow
			Dim day = rwWaregroupIncome.TageseinnahmeRow.Datum.Day
			Dim iProdGroup = Array.IndexOf(productGroups, rwPG)
			Dim income = rwWaregroupIncome.Warengruppeneinnahme / 100
			ArrProductGroupIncome(iProdGroup) += income
		Next
		For Each ProductGroupIncomeIncome In ArrProductGroupIncome
			MonthIncome += ProductGroupIncomeIncome
		Next
		For Each rwIncome In Tageseinnahme.Where(Function(x) x.Datum >= Startdate AndAlso x.Datum < dt1)
			MonthCustomers += rwIncome.Kundenzahl
		Next
		If Not MonthlyIncomeRecord Is Nothing Then
			If MonthIncome > MonthlyIncomeRecord.Item2 / 100 Then
				ShowMessageform("Monatsumsatzrekord", MonthlyIncomeRecord.Item2.ToString("#,##0.00 €"), MonthIncome.ToString("#,##0.00 €"), Datum.ToString("MMMM yyyy"))
				NewRecordFound = True
			End If
		Else
			NewRecordFound = True
		End If
		If Not MonthlyCustomersRecord Is Nothing Then
			If MonthCustomers > MonthlyCustomersRecord.Item2 Then
				ShowMessageform("Monatskundenrekord", MonthlyCustomersRecord.Item2.ToString("#,###"), MonthCustomers.ToString("#,###"), Datum.ToString("MMMM yyyy"))
				NewRecordFound = True
			End If
		Else
			NewRecordFound = True
		End If
		Dim i = -1
		For Each tpl In MonthlyGroupRecords
			i += 1
			If ArrProductGroupIncome(i) > tpl.Item3 / 100 Then
				ShowMessageform("Monatsrekord " & tpl.Item2.Name, (tpl.Item3 / 100).ToString("#,##0.00 €"), ArrProductGroupIncome(i).ToString("#,##0.00 €"), Datum.ToString("MMMM yyyy"))
				NewRecordFound = True
			End If
		Next
		If NewRecordFound Then CalculateMonthlyRecords()
	End Sub














	Partial Public Class KostenartDataTable
        Public Function CostExists(Costname As String) As Boolean
            Return Me.Any(Function(z) z.Name = Costname)
        End Function
    End Class
    Partial Public Class ArtikelDataTable
        Public Function ArtNrExistsWithinASupplier(ArtNr As String, Supplier As LieferantRow) As Boolean
            Dim ArticleListBySupplier = Supplier.GetArtikelRows
            For Each article In ArticleListBySupplier
                If article.ArtNr = ArtNr Then Return True
            Next
            Return False
        End Function
        Public Function EANExists(EAN As String) As Boolean
            If EAN.IsEmpty Then Return False
            Return Me.Any(Function(z) z.EAN = EAN)
        End Function
    End Class
    Partial Public Class KategorieDataTable
        Public Function CategoryExists(CategoryName As String) As Boolean
            Return Me.Any(Function(z) z.Name = CategoryName)
        End Function
    End Class
    Partial Public Class WarengruppeDataTable
        Public Function ProductGroupExists(ProductGroupName As String) As Boolean
            Return Me.Any(Function(z) z.Name = ProductGroupName)
        End Function
    End Class
    Partial Public Class LieferantDataTable
        Public Function SupplierExists(SupplierName As String) As Boolean
            Return Me.Any(Function(z) z.Name = SupplierName)
        End Function
        Public Function FindSupplierByName(Suppliername As String) As DtsDaten.LieferantRow
            Return Me.FirstOrDefault(Function(z) z.Name = Suppliername)
        End Function



    End Class
End Class
