﻿Imports System.IO
Imports OfficeOpenXml
Imports OfficeOpenXml.FormulaParsing.Excel.Functions.RefAndLookup

Public Class Export
    Public Event ExportStartet(Text As String)
    Public Event ExportFinished()
    Public ReadOnly ProgressReporter As New Progress(Of Integer)
    Private ReadOnly AbstractProgressReporter As IProgress(Of Integer) = ProgressReporter

    Private _DGVToExport As DataGridView
    Private _BindingSource As BindingSource
    Private _Dts As DtsDaten
    Public Property _Cancel As Boolean
    Public Sub New(DGVToExport As DataGridView)
        _DGVToExport = DGVToExport
        If DGVToExport Is Nothing Then Return
        _BindingSource = DirectCast(_DGVToExport.DataSource, BindingSource)
        _Dts = DirectCast(DirectCast(_BindingSource.DataSource, BindingSource).DataSource, DtsDaten)
    End Sub
    Public Async Sub CollectUserDataForDGVExport()
        Dim ExportInfos = GetColumnsToExport()
        Dim FilePath = GetFilePathFromUser(ExportInfos.Exportcsv)
        Dim UserInputCheck = UserSelectionCorrect(ExportInfos, FilePath)
        If Not UserInputCheck.Correct Then
            ShowAutoClosingMessageBox(UserInputCheck.Errortext)
            Return
        End If
        If ExportInfos.Exportcsv Then
            RaiseEvent ExportStartet("csv Export läuft")
            Await Task.Run(Sub() StartDGVtoCSVExport(ExportInfos, FilePath))
            RaiseEvent ExportFinished()
        Else
            RaiseEvent ExportStartet("xlsx Export läuft")
            Await Task.Run(Sub() StartDGVtoXLSXExport(ExportInfos, FilePath))
            RaiseEvent ExportFinished()
        End If
    End Sub
    Private Function GetColumnsToExport() As (ColumnsToExport As List(Of DataColumn), Exportcsv As Boolean, Delimiter As String)
        Dim AllColumns As New List(Of DataColumn)
        Dim ExportColumns As New List(Of DataColumn)
        Dim Exportcsv As Boolean
        Dim Delimiter As String

        For Each col As DataColumn In _BindingSource.DataTable.Columns
            If Not col.ColumnName = "ID" Then AllColumns.Add(col)
        Next
        Using ColChooseForm As New dlgSpaltenAuswahl
            With ColChooseForm
                ._AllColumns = AllColumns
                If .ShowDialog() = DialogResult.OK Then
                    ExportColumns = ._ExportColumns
                    Exportcsv = ._Exportcsv
                    Delimiter = ._Delimiter
                End If
            End With
        End Using
        Return (ExportColumns, Exportcsv, Delimiter)
    End Function
    Private Function GetFilePathFromUser(CreateCsv As Boolean) As String
        Using SFD As New SaveFileDialog()
            If CreateCsv Then
                SFD.Filter = "csv Dateien (*.csv)|*.csv"
            Else
                SFD.Filter = "xlsx Dateien (*.xlsx)|*.xlsx"
            End If
            SFD.InitialDirectory = My.Computer.FileSystem.SpecialDirectories.Desktop

            If SFD.ShowDialog() = DialogResult.OK Then
                Return SFD.FileName
            Else
                Return ""
            End If
        End Using
    End Function
    Private Function UserSelectionCorrect(ExportInfos As (ColumnsToExport As List(Of DataColumn), Exportcsv As Boolean, Delimiter As String), Filepath As String) As (Correct As Boolean, Errortext As String)
        If ExportInfos.ColumnsToExport.Count = 0 Then Return (False, "keine Exportspalten ausgewählt")
        If ExportInfos.Exportcsv AndAlso ExportInfos.Delimiter.IsEmpty Then Return (False, "kein csv Trennzeichen festgelegt")
        If Filepath.IsEmpty Then Return (False, "kein Dateinamen angegeben")
        Return (True, "")
    End Function
    Private Sub StartDGVtoCSVExport(ExportInfos As (ColumnsToExport As List(Of DataColumn), Exportcsv As Boolean, Delimiter As String), Filepath As String)
        Dim ColumnsToExport As List(Of DataColumn) = ExportInfos.ColumnsToExport
        Dim Delimiter As String = ExportInfos.Delimiter
        Dim csvFile As String = ""

        For i = 0 To ColumnsToExport.Count - 1
            Dim Colname = ColumnsToExport(i).ColumnName
            If Colname.Contains("ID") Then Colname = Colname.Replace("ID", "")
            csvFile &= Colname & Delimiter
        Next
        csvFile &= Environment.NewLine

        Dim ArticleAmount = _BindingSource.Count - 1
        For i = 0 To ArticleAmount
            If _Cancel Then Return
            For Each col In ColumnsToExport
                csvFile &= GetCellValue(i, col, Delimiter).ToString & Delimiter
            Next
            csvFile &= Environment.NewLine
            AbstractProgressReporter.Report(CInt((i + 1) * 100 / ArticleAmount))
        Next

        File.WriteAllText(Filepath, csvFile)
    End Sub
    Private Sub StartDGVtoXLSXExport(ExportInfos As (ColumnsToExport As List(Of DataColumn), Exportcsv As Boolean, Delimiter As String), Filepath As String)
        Dim ColumnsToExport As List(Of DataColumn) = ExportInfos.ColumnsToExport
        Dim RowCounter = 0

        Using ExcelPackage As New ExcelPackage()

            ExcelPackage.Workbook.Properties.Created = DateTime.Now
            Dim worksheet As ExcelWorksheet = ExcelPackage.Workbook.Worksheets.Add("Seite 1")
            Dim ArticleAmount = _BindingSource.Count - 1

            WriteHeader(ColumnsToExport, worksheet, ArticleAmount)

            RowCounter = WriteData(ColumnsToExport, worksheet, ArticleAmount)

            FormatColumns(ColumnsToExport, worksheet, RowCounter)

            Dim fi As FileInfo = New FileInfo(Filepath)
            ExcelPackage.SaveAs(fi)

        End Using
    End Sub
    Private Sub WriteHeader(ColumnsToExport As List(Of DataColumn), worksheet As ExcelWorksheet, ArticleAmount As Integer)
        For i = 0 To ColumnsToExport.Count - 1
            Dim Colname As String = ColumnsToExport(i).ColumnName
            If Colname.Contains("ID") Then Colname = Colname.Replace("ID", "")
            worksheet.Cells(1, i + 1).Value = Colname
        Next
    End Sub
    Private Function WriteData(ColumnsToExport As List(Of DataColumn), worksheet As ExcelWorksheet, ArticleAmount As Integer) As Integer
        Dim RowCounter = 0
        Dim Colcounter = 0
        For i = 0 To ArticleAmount
            If _Cancel Then Return (RowCounter)
            RowCounter += 1
            Colcounter = 1
            For Each col In ColumnsToExport
                worksheet.Cells(i + 2, Colcounter).Value = GetCellValue(i, col, "")
                Colcounter += 1
                AbstractProgressReporter.Report(CInt((i + 1) * 100 / (ArticleAmount + 1)))
            Next
        Next
        RowCounter += 1
        Return RowCounter
    End Function
    Private Function GetCellValue(i As Integer, col As DataColumn, Delimiter As String) As Object
        Dim DataRow As DataRowView
        Dim CellValue As Object
        DataRow = DirectCast(_BindingSource.Item(i), DataRowView)

        Select Case col.ColumnName
            Case = "KategorieID"
                If Not DataRow(col.ColumnName).ToString = "" Then
                    CellValue = GetCategoryName(CInt(DataRow(col.ColumnName)))
                Else
                    CellValue = ""
                End If
            Case = "WarengruppeID"
                If Not DataRow(col.ColumnName).ToString = "" Then
                    CellValue = GetProductGroupName(CInt(DataRow(col.ColumnName)))
                Else
                    CellValue = ""
                End If
            Case = "LieferantID"
                If Not DataRow(col.ColumnName).ToString = "" Then
                    CellValue = GetSupplierName(CInt(DataRow(col.ColumnName)))
                Else
                    CellValue = ""
                End If
            Case = "VertreterID"
                If Not DataRow(col.ColumnName).ToString = "" Then
                    CellValue = GetRepresentativeName(CInt(DataRow(col.ColumnName)))
                Else
                    CellValue = ""
                End If
            Case Else
                If Not DataRow(col.ColumnName).ToString Is Nothing Then
                    CellValue = DataRow(col.ColumnName)
                Else
                    CellValue = ""
                End If
        End Select
        Return CellValue
    End Function
    Public Function GetCategoryName(ID As Integer) As String
        Dim CategoryRows = _Dts.Kategorie
        Dim FoundCategory = CategoryRows.FirstOrDefault(Function(z) z.ID = ID)
        Return FoundCategory.Name
    End Function
    Public Function GetProductGroupName(ID As Integer) As String
        Dim ProductGroupRows = _Dts.Warengruppe
        Dim FoundGroup = ProductGroupRows.FirstOrDefault(Function(z) z.ID = ID)
        Return FoundGroup.Name
    End Function
    Public Function GetSupplierName(ID As Integer) As String
        Dim SupplierRow = _Dts.Lieferant
        Dim FoundSupplier = SupplierRow.FirstOrDefault(Function(z) z.ID = ID)
        Return FoundSupplier.Name
    End Function
    Public Function GetRepresentativeName(ID As Integer) As String
        Dim AddressRow = _Dts.Adresse
        Dim FoundAddress = AddressRow.FirstOrDefault(Function(z) z.ID = ID)
        Return FoundAddress.Nachname
    End Function
    Private Sub FormatColumns(ColumnsToExport As List(Of DataColumn), worksheet As ExcelWorksheet, Rowcounter As Integer)
        For i = 0 To ColumnsToExport.Count - 1
            Select Case ColumnsToExport(i).DataType
                Case = System.Type.GetType("System.Decimal")
                    worksheet.Cells(1, i + 1, Rowcounter, i + 1).Style.Numberformat.Format = "#,##0.00"
                Case = System.Type.GetType("System.DateTime")
                    worksheet.Cells(1, i + 1, Rowcounter, i + 1).Style.Numberformat.Format = "dd.MM.yyyy"
                Case = System.Type.GetType("System.Double")
                    worksheet.Cells(1, i + 1, Rowcounter, i + 1).Style.Numberformat.Format = "0.##"
            End Select
        Next
        worksheet.Cells(worksheet.Dimension.Address).AutoFitColumns()
    End Sub
    Public Async Sub ExportCashRegisterData()
        RaiseEvent ExportStartet("Kassenexport läuft")
        Await Task.Run(Sub() WriteCSVForCashRegister())
        RaiseEvent ExportFinished()
    End Sub
    Private Sub WriteCSVForCashRegister()
        Dim charDelimiter As Char = ";"c
        Dim i As Integer
        Dim ArticleAmount = _Dts.Artikel.Count
        Dim Path = My.Computer.FileSystem.SpecialDirectories.Desktop
        Dim CompleteFileName As String = $"{Path}\{System.DateTime.Now.ToString("yyyy.MM.dd")} - Export Kasse.csv"
        Dim csvFile As String = ""

        csvFile &= "EAN" & charDelimiter
        csvFile &= "Name" & charDelimiter
        csvFile &= "Steuer" & charDelimiter
        csvFile &= "VK" & charDelimiter
        csvFile &= "Warengruppe"
        csvFile &= Environment.NewLine

        For Each row In _Dts.Artikel
            If _Cancel Then Return
            csvFile &= row.EAN & charDelimiter
            csvFile &= row.Name1 & charDelimiter
            Dim taxes As String = row.Steuer.ToString
            If taxes = "19" Then
                csvFile &= "1" & charDelimiter
            Else
                csvFile &= "2" & charDelimiter
            End If
            csvFile &= row.Verkaufspreis.ToString("#,##0.00 €") & charDelimiter
            csvFile &= row.WarengruppeRow.Nummer
            csvFile &= Environment.NewLine

            AbstractProgressReporter.Report(CInt((i + 1) * 100 / (ArticleAmount + 1)))
            i += 1
        Next

        My.Computer.FileSystem.WriteAllText(CompleteFileName, csvFile, False)
    End Sub

End Class
