﻿Imports System.ComponentModel
<ToolboxItem(True),
Description("eine DropDownList-ComboBox, deren DataBinding-Auswahl auch gelöscht werden kann")>
Public Class ClearableComboBox : Inherits ComboBox
    '''<summary>der anzuzeigende Text bei Nicht-Auswahl</summary>
    Property DefaultText As String = "(keine Auswahl getroffen)"
    <Browsable(False)> Shadows Property DropDownStyle As ComboBoxStyle
        Get
            Return MyBase.DropDownStyle
        End Get
        Set
            MyBase.DropDownStyle = ComboBoxStyle.DropDownList
        End Set
    End Property
    '''<summary>gibt wieder, ob keine Auswahl getroffen wurde</summary>
    Private _IsEmpty As Boolean = True
    Public Property IsEmpty As Boolean
        Get
            Return _IsEmpty
        End Get
        Private Set
            _IsEmpty = Value
        End Set
    End Property
    Private _DataSource As Object
    Private _ValueMember As String
    Private _DisplayMember As String
    Private _DataBindingToRemember As Binding = Nothing
    Public Sub New()
        MyBase.New
        DropDownStyle = ComboBoxStyle.DropDownList
    End Sub
    Public Sub SetMeNotEmpty()
        IsEmpty = False
    End Sub
    '''<summary>löscht die eventuell bereits getätigte Auswahl</summary>
    Public Sub Clear()
        SaveCurrentData()
        RemoveControlData()
        SetDefaultTextAsOnlyItem()
        TreatMeAsEmpty()
    End Sub
    Public Sub UndoClear()
        If _DataSource Is Nothing Then Return
        RestoreData()
        TreatMeAsFilled()
    End Sub
    Private Sub SaveCurrentData()
        If DataSource Is Nothing Then Return
        _DataSource = DataSource
        _ValueMember = ValueMember
        _DisplayMember = DisplayMember
        If DataBindings.Count > 0 Then _DataBindingToRemember = DataBindings(0)
    End Sub
    Private Sub RemoveControlData()
        DataSource = Nothing
        If DataBindings.Count > 0 Then DataBindings.RemoveAt(0)
    End Sub
    Private Sub SetDefaultTextAsOnlyItem()
        Items.Clear()
        If String.IsNullOrEmpty(DefaultText) Then Return
        Items.Add(DefaultText)
        SelectedIndex = 0
    End Sub
    Private Sub TreatMeAsEmpty()
        _IsEmpty = True
    End Sub
    Private Sub ClearableComboBox_DropDown(sender As Object, e As EventArgs) Handles Me.DropDown
        If _DataSource Is Nothing Then Return
        RestoreData()
        TreatMeAsFilled()
    End Sub
    Private Sub RestoreData()
        DataSource = _DataSource
        ValueMember = _ValueMember
        DisplayMember = _DisplayMember
        If DataBindings.Count = 0 AndAlso _DataBindingToRemember IsNot Nothing Then DataBindings.Add(_DataBindingToRemember)
    End Sub
    Private Sub TreatMeAsFilled()
        IsEmpty = False
    End Sub
End Class