﻿Imports System.IO

Public Class UCLieferungen
    Public Event OpenCustomerOrderFound(BSCurrent As Object)
    Public Event OpenReclamationFound(BSCurrent As Object)
    Public Event OrderSelected()
    Public Property _FireEvents As Boolean = True
    Private _SaveDGVColumnWidths As Boolean
    Private Const _ColumnThatContainsOrderDate As Integer = 0
    Private Const _ColumnThatContainsDeliveryDate As Integer = 1
    Private Const _ColumnThatContainsDateOfInvoiceReceive As Integer = 2
    Private Const _ColumnThatContainsInvoiceNumber As Integer = 4

    Public Sub SetSupplierToShow(Suppliername As String)
        For i = 0 To BSLieferant.Count - 1
            Dim SupplierNameFromBs = DirectCast(DirectCast(BSLieferant(i), DataRowView).Row, DtsDaten.LieferantRow).Name
            If SupplierNameFromBs = Suppliername Then
                BSLieferant.Position = i
                Exit For
            End If
        Next
    End Sub
    Private Sub DGVLieferungen_LostFocus(sender As Object, e As EventArgs) Handles DGVLieferungen.LostFocus
        _SaveDGVColumnWidths = False
    End Sub
    Private Sub DGVLieferungen_ColumnWidthChanged(sender As Object, e As DataGridViewColumnEventArgs) Handles DGVLieferungen.ColumnWidthChanged
        If _SaveDGVColumnWidths Then My.Settings.DGVLieferungenSpaltenBreite = GetDGVColumnWidthsAsSaveableString(DGVLieferungen)
    End Sub
    Private Sub DGVLieferungen_Click(sender As Object, e As EventArgs) Handles DGVLieferungen.Click
        _SaveDGVColumnWidths = True
    End Sub
    Private Sub BTN_Click(sender As Object, e As EventArgs) Handles BTNNeueLieferung.Click, BTNLieferungLoeschen.Click, BTNBestellprogrammStarten.Click,
            BTNUmsatzauswertung.Click, BTNLieferungSpeichern.Click, BTNLieferungmitBstProgramm.Click,
            BTNAuswaehlen.Click
        Select Case True
            Case sender Is BTNNeueLieferung : CreateNewOrder()
            Case sender Is BTNLieferungmitBstProgramm : CreateNewOrderAndStartOrderProgram()
            Case sender Is BTNLieferungLoeschen : DeleteOrder()
            Case sender Is BTNBestellprogrammStarten : StartOrderProgram()
            Case sender Is BTNUmsatzauswertung : ShowSalesEvaluation()
            Case sender Is BTNLieferungSpeichern : CheckUserInputAndSaveOrder()
            Case sender Is BTNAuswaehlen : SelectInvoice()
        End Select
    End Sub
    Private Sub DGVLieferungen_CellDoubleClick(sender As Object, e As DataGridViewCellEventArgs) Handles DGVLieferungen.CellDoubleClick
        Dim SelectedOrder = BSLieferung.At(Of DtsDaten.LieferungRow)
        Dim DoubleClickedColumnIndex = e.ColumnIndex

        Select Case DoubleClickedColumnIndex
            Case = _ColumnThatContainsDeliveryDate
                If SelectedOrder.IsLieferdatumNull Then SelectedOrder.Lieferdatum = System.DateTime.Now
            Case = _ColumnThatContainsDateOfInvoiceReceive
                If SelectedOrder.IsRechnungserhaltNull Then SelectedOrder.Rechnungserhalt = System.DateTime.Now
            Case = _ColumnThatContainsOrderDate : LoadOrder()
            Case = _ColumnThatContainsInvoiceNumber : LoadInvoice()
        End Select
    End Sub
    Private Sub BSLieferant_CurrentChanged(sender As Object, e As EventArgs) Handles BSLieferant.CurrentChanged
        CalculateAndWriteSales()
    End Sub
    Private Sub CreateNewOrder()
        Dim SelectedSupplier = BSLieferant.At(Of DtsDaten.LieferantRow)
        Dim Dts = DirectCast(BSMain.DataSource, DtsDaten)

        Dim NewOrder As DtsDaten.LieferungRow
        NewOrder = Dts.Lieferung.NewLieferungRow()
        NewOrder.Bestelldatum = System.DateTime.Now
        NewOrder.LieferantID = SelectedSupplier.ID
        Dts.Lieferung.Rows.Add(NewOrder)
        BSLieferung.MoveFirst()
    End Sub
    Private Sub DeleteOrder()
        If BSLieferung.Current Is Nothing Then Return
        Dim SelectedOrder = BSLieferung.At(Of DtsDaten.LieferungRow)
        If ShowMessageBoxAndAskUserYesOrNo($"Die Bestellung vom {SelectedOrder.Bestelldatum.ToString("dd.MM.yyyy")} wird gelöscht.
                Sind sie sicher?") Then BSLieferung.RemoveCurrent()
    End Sub
    Private Sub LoadOrder()
        Dim SelectedSupplier = BSLieferant.At(Of DtsDaten.LieferantRow)
        Dim SelectedOrder = BSLieferung.At(Of DtsDaten.LieferungRow)
        If SelectedOrder.IsBestelldatumNull Then Return
        Dim FilePath = $"{My.Application.Info.DirectoryPath}\Daten\Bestellungen\{SelectedSupplier.Name}\"
        If Not Directory.Exists(FilePath) Then
            ShowAutoClosingMessageBox("Ordner für Bestellungen nicht gefunden")
        End If
        Dim Filename = SelectedOrder.Bestelldatum.ToShortDateString
        Dim files() As FileInfo = New DirectoryInfo(FilePath).GetFiles(Filename + "*.*")

        Select Case files.Length
            Case = 0 : ShowAutoClosingMessageBox("Keine Bestellung gespeichert.")
            Case = 1 : Process.Start(files(0).FullName.ToString)
            Case > 1
                Using FileChoose As New frmDateiAuswahl
                    With FileChoose
                        ._ShowAllFilesInFolder = False
                        ._FilenameToShow = Filename
                        ._Path = FilePath
                        .ShowDialog()
                    End With
                End Using
        End Select
    End Sub
    Private Sub LoadInvoice()
        Dim SelectedSupplier = BSLieferant.At(Of DtsDaten.LieferantRow)
        Dim SelectedInvoice = BSLieferung.At(Of DtsDaten.LieferungRow)
        If SelectedInvoice.IsRechnungsdatumNull Then Return
        Dim FilePath = $"{My.Application.Info.DirectoryPath}\Daten\Rechnungen\{SelectedSupplier.Name}\"
        If Not Directory.Exists(FilePath) Then
            ShowAutoClosingMessageBox("Ordner für Rechnungen nicht gefunden")
        End If
        Dim OriginalInvoiceNumber = SelectedInvoice.Rechnungsnummer
        Dim InvoiceNumber = OriginalInvoiceNumber.RemoveCharacters((Path.GetInvalidFileNameChars() & Path.GetInvalidPathChars()).ToArray.Distinct.ToArray)
        Dim Filename = $"{InvoiceNumber} vom {SelectedInvoice.Rechnungsdatum.ToShortDateString}"
        Dim files() As FileInfo = New DirectoryInfo(FilePath).GetFiles(Filename + "*.*")

        Select Case files.Length
            Case = 0 : ShowAutoClosingMessageBox("Keine Rechnung gespeichert.")
            Case = 1 : Process.Start(files(0).FullName.ToString)
            Case > 1
                Using FileChoose As New frmDateiAuswahl
                    With FileChoose
                        ._ShowAllFilesInFolder = False
                        ._FilenameToShow = Filename
                        ._Path = FilePath
                        .ShowDialog()
                    End With
                End Using
        End Select
    End Sub
    Private Sub StartOrderProgram()
        ShowMessageBox("Dieser Button startet nur das Bestellprogramm")
    End Sub
    Private Sub ShowSalesEvaluation()
        ShowMessageBox("Dieser Button startet die Umsatzauswertung")
    End Sub
    Private Sub CreateNewOrderAndStartOrderProgram()
        ShowMessageBox("Dieser Button legt eine neue Bestellung an und startet das Bestellprogramm")
    End Sub
    Private Sub CheckUserInputAndSaveOrder()
        Dim UserInput = CheckUserInput()
        If Not UserInput.UserInputCorret Then
            ShowAutoClosingMessageBox(UserInput.ErrorText)
            Return
        End If
        SaveOrder()
        CalculateAndWriteSales()
        CheckForCustomerOrders()
        CheckForReclamations()
    End Sub
    Private Function CheckUserInput() As (UserInputCorret As Boolean, ErrorText As String)
        If BSLieferung.Current Is Nothing Then Return (True, "Bitte erst eine Bestellung anlegen")
        SetTBTextZeroIfEmpty(TBNet19)
        SetTBTextZeroIfEmpty(TBNet7)
        Dim net19 As Double
        If Not Double.TryParse(TBNet19.Text, net19) Then Return (False, "ungültige Netto 19% Summe")
        Dim net7 As Double
        If Not Double.TryParse(TBNet7.Text, net7) Then Return (False, "ungültige Netto 7% Summe")

        Return (True, "")
    End Function
    Private Sub SetTBTextZeroIfEmpty(TB As TextBox)
        If TB.Text.IsEmpty Then TB.Text = "0"
    End Sub
    Private Sub CheckForCustomerOrders()
        If OpenCostumerOrdersPresent() Then
            If _FireEvents Then
                If ShowMessageBoxAndAskUserYesOrNo("Es wurden offene Kundenbestellungen gefunden." & Environment.NewLine &
                         "sollen diese angezeigt werden?") Then
                    RaiseEvent OpenCustomerOrderFound(BSLieferant.Current)
                End If
            Else
                ShowAutoClosingMessageBox("Es gibt offene Kundenbestellungen")
            End If
        End If
    End Sub
    Private Function OpenCostumerOrdersPresent() As Boolean
        Dim SelectedSupplier = BSLieferant.At(Of DtsDaten.LieferantRow)
        Dim OpenCustomerOrders = SelectedSupplier.GetKundenbestellungRows.FirstOrDefault(Function(z) Not z.IsDatumBestelltNull AndAlso z.IsDatumAngerufenNull)
        Return OpenCustomerOrders IsNot Nothing
    End Function
    Private Sub CheckForReclamations()
        If OpenReclamationsPresent() Then
            If _FireEvents Then
                If ShowMessageBoxAndAskUserYesOrNo("Es wurden offene Reklamationen gefunden." & Environment.NewLine &
                         "sollen diese angezeigt werden?") Then
                    RaiseEvent OpenReclamationFound(BSLieferant.Current)
                End If
            Else
                ShowAutoClosingMessageBox("Es gibt offene Reklamationen")
            End If
        End If
    End Sub
    Private Function OpenReclamationsPresent() As Boolean
        Dim SelectedSupplier = BSLieferant.At(Of DtsDaten.LieferantRow)
        Dim OpenReclamations = SelectedSupplier.GetReklamationRows.FirstOrDefault(Function(z) z.IsDatumErledigtNull)
        Return OpenReclamations IsNot Nothing
    End Function
    Private Sub SelectInvoice()
        RaiseEvent OrderSelected()
    End Sub
    Private Sub CalculateAndWriteSales()
        Dim ThisYear = System.DateTime.Now.Year
        Dim LastYear = ThisYear - 1
        Dim SalesThisYear = CalculateYearlySales(ThisYear)
        Dim SalesLastYear = CalculateYearlySales(LastYear)
        WriteSales(SalesThisYear, SalesLastYear, ThisYear, LastYear)
    End Sub
    Private Function CalculateYearlySales(year As Integer) As Double
        Dim SelectedSupplier = BSLieferant.At(Of DtsDaten.LieferantRow)
        Dim YearlyOrders = SelectedSupplier.GetLieferungRows.Where(Function(z) Not z.IsRechnungsdatumNull AndAlso z.Rechnungsdatum.Year = year)

        Dim TotalAmount As Double
        For Each order In YearlyOrders
            TotalAmount += order.Netto19 + order.Netto7
        Next
        Return TotalAmount
    End Function
    Private Sub WriteSales(SalesThisYear As Double, SalesLastYear As Double, ThisYear As Integer, LastYear As Integer)
        LBLUmsatzDiesesJahrText.Text = $"Umsatz {ThisYear}"
        LBLUmsatzDiesesJahr.Text = SalesThisYear.ToString("#,##0.00 €")
        LBLUmsatzLetztesJahrText.Text = $"Umsatz {LastYear}"
        LBLUmsatzLetztesJahr.Text = SalesLastYear.ToString("#,##0.00 €")
    End Sub
    Private Sub SaveOrder()
        Dim SelectedOrder = BSLieferung.At(Of DtsDaten.LieferungRow)
        Dim SelectedSupplier = BSLieferant.At(Of DtsDaten.LieferantRow)
        Dim Orderdate = DTPBestelldatum.Value.Date
        Dim DeliverDateIsSet = NDTPLieferdatum.Checked
        Dim DeliverDate As Date
        Dim DeliverDuration As Double
        If DeliverDateIsSet Then
            DeliverDate = CDate(NDTPLieferdatum.Value).Date
            DeliverDuration = DeliverDate.Subtract(Orderdate).TotalDays
        End If
        Dim InvoiceReceivedDateIsSet = NDTPRechnungserhalt.Checked
        Dim DateofInvoiceReceive As Date
        If InvoiceReceivedDateIsSet Then DateofInvoiceReceive = CDate(NDTPRechnungserhalt.Value).Date
        Dim InvoiceDateIsSet As Boolean = NDTPRechnungsdatum.Checked
        Dim InvoiceDate As Date
        If InvoiceDateIsSet Then InvoiceDate = CDate(NDTPRechnungsdatum.Value).Date
        Dim InvoiceNumber = TBRechnungsnummer.Text
        Dim net19 As Double = Double.Parse(TBNet19.Text)
        Dim net7 As Double = Double.Parse(TBNet7.Text)
        Dim TotalInvoiceAmount As Double
        If SelectedSupplier.MwstAusrechnen Then
            TotalInvoiceAmount = (net19 * 1.19) + (net7 * 1.07)
        Else
            TotalInvoiceAmount = net19 + net7
        End If
        Dim Note = TBNotiz.Text

        With SelectedOrder
            .Bestelldatum = Orderdate
            If DeliverDateIsSet Then
                .Lieferdatum = DeliverDate
                .Lieferzeit = DeliverDuration
            Else
                .SetLieferdatumNull()
                .SetLieferzeitNull()
            End If
            If InvoiceReceivedDateIsSet Then
                .Rechnungserhalt = DateofInvoiceReceive
            Else
                .SetRechnungserhaltNull()
            End If
            If InvoiceDateIsSet Then
                .Rechnungsdatum = InvoiceDate
            Else
                .SetRechnungsdatumNull()
            End If
            .Rechnungsnummer = InvoiceNumber
            .Netto19 = net19
            .Netto7 = net7
            .Gesamt = TotalInvoiceAmount
            .Notiz = Note
        End With
    End Sub
    Private Sub LBLDate_Click(sender As Object, e As EventArgs) Handles LBLRechnungsdatum.Click, LBLLieferdatum.Click,
        LBLDatumRechnungserhalt.Click, LBLBestelldatum.Click
        Dim TodayDate = System.DateTime.Now
        Select Case True
            Case sender Is LBLBestelldatum : SetDate(TodayDate, DTPBestelldatum, NDTPLieferdatum)
            Case sender Is LBLLieferdatum : SetDate(TodayDate, NDTPLieferdatum, NDTPRechnungserhalt)
            Case sender Is LBLDatumRechnungserhalt : SetDate(TodayDate, NDTPRechnungserhalt, NDTPRechnungsdatum)
            Case sender Is LBLRechnungsdatum : SetDate(TodayDate, NDTPRechnungsdatum, TBRechnungsnummer)
        End Select
    End Sub
    Private Sub LBLDate_DoubleClick(sender As Object, e As EventArgs) Handles LBLRechnungsdatum.DoubleClick,
        LBLLieferdatum.DoubleClick, LBLDatumRechnungserhalt.DoubleClick, LBLBestelldatum.DoubleClick
        Dim YesterdayDate = System.DateTime.Now.AddDays(-1)
        Select Case True
            Case sender Is LBLBestelldatum : SetDate(YesterdayDate, DTPBestelldatum, NDTPLieferdatum)
            Case sender Is LBLLieferdatum : SetDate(YesterdayDate, NDTPLieferdatum, NDTPRechnungserhalt)
            Case sender Is LBLDatumRechnungserhalt : SetDate(YesterdayDate, NDTPRechnungserhalt, NDTPRechnungsdatum)
            Case sender Is LBLRechnungsdatum : SetDate(YesterdayDate, NDTPRechnungsdatum, TBRechnungsnummer)
        End Select
    End Sub
    Private Sub SetDate(SelectedDate As Date, DTP As DateTimePicker, ControlToSelect As Control)
        If SelectedDate.DayOfWeek = 0 Then SelectedDate.AddDays(-1)
        DTP.Value = SelectedDate
        ControlToSelect.Focus()
    End Sub
End Class
