﻿Imports System.IO
Public Class UCReklamation
    Private Const _ColumnThatContainsReclaFinishDate As Integer = 1
    Private Const _ColumnThatContainsSupplierName As Integer = 2

    Private _SaveDGVColumnWidths As Boolean
    Private _BSFilteredBySupplierIsActivated As Boolean = False
    Public Async Sub SetSupplierToShow(BSCurrent As Object)
        For i = 0 To BSLieferant.Count - 1
            If DirectCast(BSLieferant.Item(i), Data.DataRowView).Row.Equals(DirectCast(BSCurrent, Data.DataRowView).Row) Then
                _BSFilteredBySupplierIsActivated = True
                SetBindingSource()
                Await Task.Delay(1)
                BSLieferant.Position = i
                Exit For
            End If
        Next
    End Sub
    Public Async Sub SetSupplierToShowByName(Suppliername As String)
        For i = 0 To BSLieferant.Count - 1
            Dim SupplierNameFromBs = DirectCast(DirectCast(BSLieferant(i), DataRowView).Row, DtsDaten.LieferantRow).Name
            If SupplierNameFromBs = Suppliername Then
                _BSFilteredBySupplierIsActivated = True
                SetBindingSource()
                Await Task.Delay(1)
                BSLieferant.Position = i
                Exit For
            End If
        Next
    End Sub
    Private Sub DGVReklamation_LostFocus(sender As Object, e As EventArgs) Handles DGVReklamation.LostFocus
        _SaveDGVColumnWidths = False
    End Sub
    Private Sub DGVReklamation_ColumnWidthChanged(sender As Object, e As DataGridViewColumnEventArgs) Handles DGVReklamation.ColumnWidthChanged
        If _SaveDGVColumnWidths Then My.Settings.DGVReklamationSpaltenBreite = GetDGVColumnWidthsAsSaveableString(DGVReklamation)
    End Sub
    Private Sub DGVReklamation_Click(sender As Object, e As EventArgs) Handles DGVReklamation.Click
        _SaveDGVColumnWidths = True
    End Sub
    Private Sub DGVReklamation_CellDoubleClick(sender As Object, e As DataGridViewCellEventArgs) Handles DGVReklamation.CellDoubleClick
        Dim SelectedReclamation = BSReklamation.At(Of DtsDaten.ReklamationRow)
        Dim DoubleClickedColumnIndex = e.ColumnIndex

        Select Case DoubleClickedColumnIndex
            Case = _ColumnThatContainsReclaFinishDate
                If SelectedReclamation.IsDatumErledigtNull Then SelectedReclamation.DatumErledigt = System.DateTime.Now
            Case = _ColumnThatContainsSupplierName : ShowReclamationFile()
            Case Else
                EditReclamation()
        End Select
    End Sub
    Private Sub BTNClick_Click(sender As Object, e As EventArgs) Handles BTNReklamationNeu.Click, BTNReklamationsDateiAnzeigen.Click,
            BTNReklamationBearbeiten.Click, BTNReklamationLoeschen.Click, BTNClearCCB.Click
        Select Case True
            Case sender Is BTNReklamationNeu : CreateNewReclamation()
            Case sender Is BTNReklamationsDateiAnzeigen : ShowReclamationFile()
            Case sender Is BTNReklamationBearbeiten : EditReclamation()
            Case sender Is BTNReklamationLoeschen : DeleteReclamation()
            Case sender Is BTNClearCCB : ClearClearableComboBox()
        End Select
    End Sub
    Private Sub CreateNewReclamation()
        BSReklamation.EditNew(Of dlgReklamationBearbeiten)
    End Sub
    Private Sub ShowReclamationFile()
        If BSReklamation.Current Is Nothing Then Return
        Dim SelectedReclamation = BSReklamation.At(Of DtsDaten.ReklamationRow)
        Dim Filepath = SelectedReclamation.Dateipfad
        If Filepath.IsEmpty Then
            ShowAutoClosingMessageBox("Keine Datei gespeichert.")
            Return
        End If
        Dim ReclamationFile As FileInfo = New FileInfo(My.Application.Info.DirectoryPath & "\Daten\Reklamationen\" & Filepath)
        If ReclamationFile.Exists Then
            Process.Start(ReclamationFile.FullName)
        Else
            ShowAutoClosingMessageBox("Die Reklamationsdatei wurde nicht gefunden.")
        End If
    End Sub
    Private Sub EditReclamation()
        If BSReklamation.Current Is Nothing Then Return
        BSReklamation.EditCurrent(Of dlgReklamationBearbeiten)
    End Sub
    Private Sub DeleteReclamation()
        Dim Source = GetUsedBindingsource()
        If Source.Current Is Nothing Then Return
        Dim SelectedReclamation = Source.At(Of DtsDaten.ReklamationRow)
        Dim ReclamationFilePath = SelectedReclamation.Dateipfad
        If ShowMessageBoxAndAskUserYesOrNo($"Die Reklamation von {SelectedReclamation.EXPLieferant} vom {SelectedReclamation.DatumReklamation.ToString("dd.MM.yyyy")} wird gelöscht.
                Sind sie sicher?") Then
            Source.RemoveCurrent()
            If ReclamationFilePath.IsNotEmpty Then
                If ShowMessageBoxAndAskUserYesOrNo($"Zu dieser Reklamation war eine Datei hinterlegt.{Environment.NewLine}Soll diese ebenfalls gelöscht werden?") Then
                    Dim ReclamationFile As FileInfo = New FileInfo($"{My.Application.Info.DirectoryPath}\Daten\Reklamationen\{ReclamationFilePath}")
                    If ReclamationFile.Exists Then ReclamationFile.Delete()
                End If
            End If
            End If
    End Sub
    Private Sub ClearClearableComboBox()
        CCBLieferant.Clear()
        _BSFilteredBySupplierIsActivated = Not CCBLieferant.IsEmpty
        SetBindingSource()
    End Sub
    Private Function GetUsedBindingsource() As BindingSource
        If _BSFilteredBySupplierIsActivated Then
            Return BSReklamationFiltered
        Else
            Return BSReklamation
        End If
    End Function
    Private Sub SetBindingSource()
        If _BSFilteredBySupplierIsActivated Then
            DGVReklamation.DataSource = BSReklamationFiltered
        Else
            DGVReklamation.DataSource = BSReklamation
        End If
    End Sub
    Private Sub CCBLieferant_SelectedValueChanged(sender As Object, e As EventArgs) Handles CCBLieferant.SelectedValueChanged
        _BSFilteredBySupplierIsActivated = Not CCBLieferant.IsEmpty
        SetBindingSource()
    End Sub
    Protected Overrides Function ProcessCmdKey(ByRef msg As Message, ByVal keyData As Keys) As Boolean
        Select Case keyData
            Case = Keys.Control Or Keys.N : CreateNewReclamation()
            Case = Keys.Enter : EditReclamation()
            Case = Keys.Delete : DeleteReclamation()
            Case = Keys.Escape : ClearClearableComboBox()
            Case Else : Return MyBase.ProcessCmdKey(msg, keyData)
        End Select
    End Function
End Class
