﻿Imports System.IO

Partial Class frmHaupt
    Private Sub frmHaupt_DragEnter(sender As System.Object, e As System.Windows.Forms.DragEventArgs) Handles Me.DragEnter
        If e.Data.GetDataPresent(DataFormats.FileDrop) Then
            e.Effect = DragDropEffects.Copy
        End If
    End Sub
    Private Sub frmHaupt_DragDrop(sender As System.Object, e As System.Windows.Forms.DragEventArgs) Handles Me.DragDrop
        Dim droppedFiles() As String = CType(e.Data.GetData(DataFormats.FileDrop), String())
        processDroppedFiles(droppedFiles, False)
    End Sub
    Public Sub processDroppedFiles(droppedFiles() As String, DeleteFileSelected As Boolean)

        For Each File In droppedFiles
            Dim DroppedFileInfo As FileInfo = New FileInfo(File)
            Dim NewFileName As String
            Dim DeleteOriginalFile As Boolean

            Using ChooseFileType As New dlgDateiTypAuswahl
                ChooseFileType.LBLDateiname.Text = Path.GetFileNameWithoutExtension(DroppedFileInfo.Name)
                ChooseFileType._DeleteFileCheckBoxChecked = DeleteFileSelected
                ChooseFileType._Fileinfo = DroppedFileInfo
                If My.Settings.frmDateiAuswahlPosition.IsEmpty Then
                    ChooseFileType.Size = New System.Drawing.Size(1049, 579)
                Else
                    ChooseFileType.Size = My.Settings.frmDateiAuswahlSize
                    ChooseFileType.Location = My.Settings.frmDateiAuswahlPosition
                End If
                If ChooseFileType.ShowDialog() = DialogResult.OK Then
                    DeleteOriginalFile = ChooseFileType._DeleteOriginalFile
                    Select Case ChooseFileType._FileType
                        Case = dlgDateiTypAuswahl.FileType.Invoice
                            InvoiceDropped(DroppedFileInfo, DeleteOriginalFile)
                        Case = dlgDateiTypAuswahl.FileType.Order
                            OrderDropped(DroppedFileInfo, DeleteOriginalFile)
                        Case = dlgDateiTypAuswahl.FileType.Reclamation
                            ReclamationDropped(DroppedFileInfo, DeleteOriginalFile)
                        Case = dlgDateiTypAuswahl.FileType.Costs
                            NewFileName = ChooseFileType._NewFileName
                            CostDropped(DroppedFileInfo, NewFileName, DeleteOriginalFile)
                        Case = dlgDateiTypAuswahl.FileType.Offer
                            NewFileName = ChooseFileType._NewFileName
                            ListDropped(DroppedFileInfo, NewFileName, My.Application.Info.DirectoryPath & "\Daten\Angebote\", "Angebot", DeleteOriginalFile)
                        Case = dlgDateiTypAuswahl.FileType.StockList
                            NewFileName = ChooseFileType._NewFileName
                            ListDropped(DroppedFileInfo, NewFileName, My.Application.Info.DirectoryPath & "\Daten\Stocklisten\", "Stockliste", DeleteOriginalFile)
                        Case = dlgDateiTypAuswahl.FileType.PriceList
                            NewFileName = ChooseFileType._NewFileName
                            ListDropped(DroppedFileInfo, NewFileName, My.Application.Info.DirectoryPath & "\Daten\Preislisten\", "Preisliste", DeleteOriginalFile)
                        Case = dlgDateiTypAuswahl.FileType.Open
                            ShowFileInWebView(DroppedFileInfo)
                        Case = dlgDateiTypAuswahl.FileType.Skip
                            SkipDroppedFile(DroppedFileInfo, DeleteOriginalFile)
                    End Select
                Else
                    Return
                End If
            End Using
        Next
    End Sub
    Private Sub OrderDropped(DroppedFileInfo As FileInfo, DeleteOriginalFile As Boolean)
        Dim SupplierData = GetSupplierData(DroppedFileInfo)
        If SupplierData.SelectedInvoice Is Nothing Then Return
        Dim OrderFileData = GetOrderFileDataFrom(DroppedFileInfo, SupplierData)
        StoreFileInAppFileSystem(OrderFileData, DroppedFileInfo, DeleteOriginalFile)
    End Sub
    Private Function GetSupplierData(DroppedFileInfo As FileInfo) As (Suppliername As String, SelectedInvoice As DtsDaten.LieferungRow)
        Dim SelectedSupplierPosition As Integer = SelectSupplierByPosition()
        If SelectedSupplierPosition = -1 Then Return Nothing
        Using InvoiceSelect As New dlgRechnungsAuswahl
            With InvoiceSelect
                .UcLieferungen.BSMain.DataSource = DtsDaten
                .UcLieferungen.BSLieferant.Position = SelectedSupplierPosition
                .UcLieferungen._FireEvents = False
                If PossibleToShowFileInWebView(DroppedFileInfo) Then
                    .WebView2.Source = New Uri(DroppedFileInfo.FullName)
                Else
                    ShowAutoClosingMessageBox("Die Datei kann nicht angezeigt werden.")
                End If
                If .ShowDialog = DialogResult.OK Then Return (._Suppliername, ._SelectedInvoice)
            End With
        End Using
        Return Nothing
    End Function
    Private Function GetOrderFileDataFrom(DroppedFileInfo As FileInfo, SupplierData As (Suppliername As String, SelectedInvoice As DtsDaten.LieferungRow)) As (Targetpath As DirectoryInfo, NewFileName As String, NewFileNameWithoutExtension As String, SuccessText As String)
        Dim Targetpath As DirectoryInfo
        Dim NewFileName As String
        Dim NewFileNameWithoutExtension As String
        Dim SuccessText As String
        Targetpath = New DirectoryInfo($"{My.Application.Info.DirectoryPath}\Daten\Bestellungen\{SupplierData.Suppliername}")
        NewFileNameWithoutExtension = $"{SupplierData.SelectedInvoice.Bestelldatum.ToShortDateString}"
        NewFileName = $"{NewFileNameWithoutExtension}{DroppedFileInfo.Extension}"
        SuccessText = $"Bestellung vom {SupplierData.SelectedInvoice.Bestelldatum.ToShortDateString} von {SupplierData.Suppliername} gespeichert."
        Return (Targetpath, NewFileName, NewFileNameWithoutExtension, SuccessText)
    End Function
    Private Sub StoreFileInAppFileSystem(FileData As (Targetpath As DirectoryInfo, NewFileName As String, NewFileNameWithoutExtension As String, SuccessText As String), DroppedFileInfo As FileInfo, DeleteOriginalFile As Boolean)
        Directory.CreateDirectory(FileData.Targetpath.FullName)
        Dim NewFullFilePath = $"{FileData.Targetpath.FullName}\{FileData.NewFileName}"
        Do While File.Exists(NewFullFilePath)
            If Not ShowMessageBoxAndAskUserYesOrNo($"Die Datei {FileData.NewFileName} existiert bereits.{Environment.NewLine}Wollen Sie {FileData.NewFileName} umbennen?") Then Exit Do
            FileData.NewFileName = SetNewFilename(FileData.NewFileName, DroppedFileInfo.Extension, FileData.NewFileNameWithoutExtension)
            NewFullFilePath = $"{FileData.Targetpath.FullName}\{FileData.NewFileName}"
            If FileData.NewFileName.IsEmpty Then Return
        Loop
        DroppedFileInfo.CopyTo(NewFullFilePath, True)
        ShowAutoClosingMessageBox(FileData.SuccessText)
        If DeleteOriginalFile Then DroppedFileInfo.Delete()
    End Sub
    Private Sub InvoiceDropped(DroppedFileInfo As FileInfo, DeleteOriginalFile As Boolean)
        Dim SupplierData = GetSupplierData(DroppedFileInfo)
        If SupplierData.SelectedInvoice Is Nothing Then Return
        Dim InvoiceFileData = GetInvoiceFileDataFrom(DroppedFileInfo, SupplierData)
        If InvoiceFileData.Targetpath Is Nothing Then Return
        StoreFileInAppFileSystem(InvoiceFileData, DroppedFileInfo, DeleteOriginalFile)
    End Sub
    Private Function GetInvoiceFileDataFrom(DroppedFileInfo As FileInfo, SupplierData As (Suppliername As String, SelectedInvoice As DtsDaten.LieferungRow)) As (Targetpath As DirectoryInfo, NewFileName As String, NewFileNameWithoutExtension As String, SuccessText As String)
        Dim Targetpath As DirectoryInfo
        Dim NewFileName As String
        Dim NewFileNameWithoutExtension As String
        Dim SuccessText As String
        Targetpath = New DirectoryInfo($"{My.Application.Info.DirectoryPath}\Daten\Rechnungen\{SupplierData.Suppliername}")
        If SupplierData.SelectedInvoice.IsRechnungsdatumNull OrElse SupplierData.SelectedInvoice.Rechnungsnummer.IsEmpty Then
            ShowAutoClosingMessageBox("ungültige Rechnung ausgewählt")
            Return Nothing
        End If
        Dim OriginalInvoiceNumber = SupplierData.SelectedInvoice.Rechnungsnummer
        Dim InvoiceNumber = OriginalInvoiceNumber.RemoveCharacters((Path.GetInvalidFileNameChars() & Path.GetInvalidPathChars()).ToArray.Distinct.ToArray)
        NewFileNameWithoutExtension = $"{InvoiceNumber} vom {SupplierData.SelectedInvoice.Rechnungsdatum.ToShortDateString}"
        NewFileName = $"{NewFileNameWithoutExtension}{DroppedFileInfo.Extension}"
        SuccessText = $"Rechnung {OriginalInvoiceNumber} vom {SupplierData.SelectedInvoice.Rechnungsdatum.ToShortDateString} von {SupplierData.Suppliername} gespeichert."
        Return (Targetpath, NewFileName, NewFileNameWithoutExtension, SuccessText)
    End Function
    Private Sub ReclamationDropped(DroppedFileInfo As FileInfo, DeleteOriginalFile As Boolean)
        Dim SupplierID = SelectSupplierByID()
        If SupplierID = 1 Then Return

        Dim tpls As New List(Of Tuple(Of DataColumn, Object))
        With DtsDaten.Reklamation
            tpls.Add(Tuple.Create(Of DataColumn, Object)(.DatumReklamationColumn, System.DateTime.Now))
            tpls.Add(Tuple.Create(Of DataColumn, Object)(.LieferantIDColumn, SupplierID))
            tpls.Add(Tuple.Create(Of DataColumn, Object)(.DateipfadColumn, DroppedFileInfo.FullName))
            BSReklamation.EditNew(Of dlgReklamationBearbeiten)(tpls)
        End With

        CheckIfReclamationCanBeSendByMail(BSReklamation.At(Of DtsDaten.ReklamationRow))
        If DeleteOriginalFile Then DroppedFileInfo.Delete()
    End Sub
    Private Sub CheckIfReclamationCanBeSendByMail(SelectedReclamation As DtsDaten.ReklamationRow)
        If SelectedReclamation.ReklamiertVia = "Mail" Then
            If ShowMessageBoxAndAskUserYesOrNo("Wollen Sie die Reklamation per Mail versenden?") Then
                SendReclamationViaMail(SelectedReclamation)
            End If
        End If
    End Sub
    Private Sub SendReclamationViaMail(SelectedReclamation As DtsDaten.ReklamationRow)
        Dim SelectedSupplier = SelectedReclamation.LieferantRow
        Dim Attachment As New List(Of String)
        Dim RecipientAddress As String
        Attachment.Add($"{My.Application.Info.DirectoryPath}\Daten\Reklamationen\{SelectedReclamation.Dateipfad}")
        If SelectedReclamation.ReklamiertBei = "Vertreter" Then
            If Not SelectedSupplier.AdresseRow Is Nothing Then RecipientAddress = SelectedSupplier.AdresseRow.Mail1
        Else
            RecipientAddress = SelectedSupplier.Email
        End If
        Dim sendmail = New frmEmailDaten
        With sendmail
            .BSMain.DataSource = DtsDaten
            .SetRecipient = RecipientAddress
            .GetAttachments = Attachment
            .TBBetreff.Select()
            If sendmail.ShowDialog() = DialogResult.OK Then
                ShowAutoClosingMessageBox("Email wird versendet")
            Else
                ShowAutoClosingMessageBox("Emailversand abgebrochen")
            End If
        End With
    End Sub
    Private Sub CostDropped(DroppedFileInfo As FileInfo, NewFileName As String, DeleteOriginalFile As Boolean)
        Dim SelectedCostName = ChooseCostType()
        If SelectedCostName.IsEmpty Then Return
        StoreCostFileInAppFileSystem(DroppedFileInfo, NewFileName, SelectedCostName, DeleteOriginalFile)
    End Sub
    Private Sub StoreCostFileInAppFileSystem(DroppedFileInfo As FileInfo, NewFileName As String, SelectedCostName As String, DeleteOriginalFile As Boolean)
        Dim Targetpath = New DirectoryInfo($"{My.Application.Info.DirectoryPath}\Daten\Kosten\{SelectedCostName}")
        Dim InAppFullFilePath = SetInAppFilePath(Targetpath, NewFileName)

        Do While File.Exists(InAppFullFilePath)
            If Not ShowMessageBoxAndAskUserYesOrNo($"Die Datei {NewFileName} existiert bereits.{Environment.NewLine}Wollen Sie {NewFileName} umbennen?") Then Exit Do
            NewFileName = SetNewFilename(NewFileName, DroppedFileInfo.Extension)
            InAppFullFilePath = $"{Targetpath.FullName}\{NewFileName}"
            If NewFileName.IsEmpty Then Return
        Loop
        DroppedFileInfo.CopyTo(InAppFullFilePath, True)
        If DeleteOriginalFile Then DroppedFileInfo.Delete()
        ShowAutoClosingMessageBox($"Kosten  -  {SelectedCostName} - {NewFileName} gespeichert")
    End Sub
    Private Function SetInAppFilePath(Targetpath As DirectoryInfo, NewFilename As String) As String
        Directory.CreateDirectory(Targetpath.FullName)
        Dim InAppFullFilePath = $"{Targetpath.FullName}\{NewFilename}"
        Return InAppFullFilePath
    End Function
    Private Function ChooseCostType() As String
        Dim SelectedCostname As String
        Using CostSelect As New dlgKostenAuswahl
            CostSelect.BSMain.DataSource = DtsDaten
            If CostSelect.ShowDialog = DialogResult.OK Then SelectedCostname = CostSelect._Costname
        End Using
        Return SelectedCostname
    End Function
    Private Sub ListDropped(DroppedFileInfo As FileInfo, NewFileName As String, CopyPath As String, ListType As String, DeleteOriginalFile As Boolean)
        Dim SupplierName = SelectSupplierByName()
        If SupplierName.IsEmpty Then Return

        Dim TargetPath = New DirectoryInfo(CopyPath & SupplierName)
        Dim InAppFullFilePath = SetInAppFilePath(TargetPath, NewFileName)

        StoreListFileInAppFileSystem(DroppedFileInfo, InAppFullFilePath, TargetPath, NewFileName, SupplierName, DeleteOriginalFile, ListType)

    End Sub
    Private Sub StoreListFileInAppFileSystem(DroppedFileInfo As FileInfo, InAppFullFilePath As String, targetpath As DirectoryInfo,
                                             NewFileName As String, Suppliername As String, DeleteOriginalFile As Boolean,
                                             ListType As String)

        Do While File.Exists(InAppFullFilePath)
            If Not ShowMessageBoxAndAskUserYesOrNo($"Die Datei {NewFileName} existiert bereits.{Environment.NewLine}Wollen Sie {NewFileName} umbennen?") Then Exit Do
            NewFileName = SetNewFilename(NewFileName, DroppedFileInfo.Extension)
            InAppFullFilePath = $"{targetpath.FullName}\{NewFileName}"
            If NewFileName.IsEmpty Then Return
        Loop

        DroppedFileInfo.CopyTo(InAppFullFilePath, True)
        ShowAutoClosingMessageBox($"{Suppliername} {ListType}  -  {NewFileName} gespeichert")

        If DeleteOriginalFile Then DroppedFileInfo.Delete()
    End Sub
    Private Sub ShowFileInWebView(DroppedFileInfo As FileInfo)
        If PossibleToShowFileInWebView(DroppedFileInfo) Then
            With UcArtikel
                .WebView2.Source = New Uri(DroppedFileInfo.FullName)
                .SCArtikel.SplitterDistance = My.Settings.SCArtikelSplitterDistance
                .SCArtikel.Panel2Collapsed = False
                ShowArticlePage()
            End With
        Else
            ShowAutoClosingMessageBox("Die Datei kann nicht angezeigt werden.")
        End If
    End Sub
    Private Sub SkipDroppedFile(DroppedFileInfo As FileInfo, DeleteOriginalFile As Boolean)
        If DeleteOriginalFile Then DroppedFileInfo.Delete()
    End Sub
End Class
